Changes for Linux:

BIG_ENDIAN conflicts with another already defined macro. Something from
BSD, I don't know. So I changed it to
	access.h:#ifdef USE_BIG_ENDIAN
	update.c:#ifdef USE_BIG_ENDIAN

The compiler complained about some string routines therefore in some sources
I change strings.h to string.h.

So I got it to work. But copying files to Linux directories didn't work. Yes,
because of the '/' being an introducer for a qualifier. So I changed cmdsplit

/* 
 * New feature for Unix: '//' stops qualifier parsing (similar to '--' 
 * for some Unix tools). This enables us to copy to Unix directories.
 * 	copy /bin // *.com /tmp/*
 * is split into argv[0] -> "*.com" argv[1] -> "/tmp/*" qualv[0] -> "/bin"
 */

Next I thought it would be nice to have a /PAGE for the directory command
and for the type command and for search. But then I thought that this is
Linux and usually there is a good pipe implementation which can do that for
me. The only thing I missed was a one-liner to start the ods2 reader with
some commands. OK, there is the undocumented '@' and there is input
redirection. But both require editing a file. For simple command sequences
it seems too much overhead. Having a one-liner in the recall buffer and
changing that is much easier. So I added command arguments which are
interpreted as chained commands to the ods2 reader.

There are no command line options, yet. So everything is accepted as a
command string(s). As an internal delimiter I chose '$'.

/*
 * Parse the command line to read and execute commands:
 * 	./ods2 mount scd1 $ set def [hartmut] $ copy *.com $ exit
 * '$' is used as command delimiter because it is a familiar character
 * in the VMS world. However, it should be used surounded by white spaces;
 * otherwise, a token '$copy' is interpreted by the Unix shell as a shell
 * variable. Quoting the whole command string might help:
 * 	./ods2 'mount scd1 $set def [hartmut] $copy *.com $exit'
 * If the ods2 reader should use any switches '-c' should be used for
 * the command strings, then quoting will be required:
 * 	./ods2 -c 'mount scd1 $ set def [hartmut] $ copy *.com $ exit'
 *
 * The same command concatenation can be implemented for the prompted input.
 * As for indirect command files (@), it isn't checked if one of the chained
 * commands fails. The user has to be careful, all the subsequent commands
 * are executed!
 */

Now I can use the ods2 reader like

./ods2 mount sda $ dir [...]/file/date $ exit |more
./ods2 mount sda $ dir [...]/file/date $ exit |grep '-2003' |more
./ods2 'mount sda: $set def [hartmut] $copy/bin *.com // /tmp/* $exit'

The single quotes are useful if I want to prevent the Linux shell from
expanding it's meta characters: '*' etc. The '/bin' is not useful for
text files like command procedure. Here it is used just to show the
effect of '//'. So the last example gives:

ODS2 V1.3hb
%MOUNT-I-MOUNTED, Volume SYS073       mounted on sda:
%COPY-S-COPIED, sda:[HARTMUT]CRECRC.COM;38 copied to /tmp/x/CRECRC.COM (59 records)
%COPY-S-COPIED, sda:[HARTMUT]JSTART.COM;4 copied to /tmp/x/JSTART.COM (4 records)
%COPY-S-COPIED, sda:[HARTMUT]LOGIN.COM;2 copied to /tmp/x/LOGIN.COM (71 records)
%COPY-S-COPIED, sda:[HARTMUT]NB.COM;5 copied to /tmp/x/NB.COM (15 records)
%COPY-S-COPIED, sda:[HARTMUT]NBR.COM;1 copied to /tmp/x/NBR.COM (3 records)
%COPY-S-COPIED, sda:[HARTMUT]OLD-NB.COM;1 copied to /tmp/x/OLD-NB.COM (15 records)
%COPY-S-COPIED, sda:[HARTMUT]PCDISK.COM;3 copied to /tmp/x/PCDISK.COM (4 records)
%COPY-S-COPIED, sda:[HARTMUT]START.COM;2 copied to /tmp/x/START.COM (3 records)
%COPY-S-COPIED, sda:[HARTMUT]U2V.COM;41 copied to /tmp/x/U2V.COM (236 records)
%COPY-S-COPIED, sda:[HARTMUT]V2U-OLD.COM;1 copied to /tmp/x/V2U-OLD.COM (36 records)
%COPY-S-COPIED, sda:[HARTMUT]V2U.COM;35 copied to /tmp/x/V2U.COM (59 records)
%COPY-S-COPIED, sda:[HARTMUT]X.COM;1 copied to /tmp/x/X.COM (91 records)
%COPY-S-COPIED, sda:[HARTMUT]XLOGIN.COM;1 copied to /tmp/x/XLOGIN.COM (89 records)
%COPY-S-NEWFILES, 13 files created

These examples always have an exit command at the end. But that's not
necessary. If there is none, the ods2 reader continues reading commands from
stdin:

./ods2 mount sda $ set def [hartmut]
 ODS2 V1.3hb
%MOUNT-I-MOUNTED, Volume SYS073       mounted on sda:
$> dir *.com
Directory sda:[HARTMUT]

CRECRC.COM;38       JSTART.COM;4        LOGIN.COM;2         NB.COM;5
NBR.COM;1           OLD-NB.COM;1        PCDISK.COM;3        START.COM;2
U2V.COM;41          V2U-OLD.COM;1       V2U.COM;35          X.COM;1
XLOGIN.COM;1

Total of 13 files.
$> exit

So on a Linux box an alias can be set up such that I end up on the VMS disk
in my directory.

I'm not happy with the (error) messages being written to stdout, they should
go to stderr, so I can redirect only the wanted output to a file. Maybe I'll
change that, later.

One note on copy/binary. It is not what one might expect. The ods2 reader
checks if the VMS file has the attributes FAB$M_CR | FAB$M_PRN | FAB$M_FTN. If
present, the read data is assumed to be text and a "\n" is appended to each
record when writing the copy. "/binary" suppresses that. If the VMS file has
none of the attributes set, the data is copied as is. Executables with fixed
length records will be copied as 512 byte records and will not differ if
copied with or without "/binary". Files with variable record length will
always be copied without the record length, regardless of "/binary". But if
"/binary" is used on a file, which has one of the above attributes set, the file
may not be copied: the ods2 reader can't handle empty records in "binary"
mode. Anyway, I was not able to copy a Java class file and run it on Linux.

On VMS there can be virtual disks (VDAnnn, LDAnnn), which make a disk
out of a file. These files, disk images, can be copied to Linux disks.
To let ods2 read these virtual disks a simple symbolic link from
/dev/diskfile to the file is set up. Then the ususal 'mount diskfile'
can be used. Sorry, the user needs the right to create the symbolic link
in /dev or root has to do that. Or losetup can be used to create a loop
device for the disk image, so a 'mount loop0' will work. Again, the
needs the right to do that.


Diff of what was changed:

diff -u o/access.h ./access.h
--- o/access.h	Fri Sep  7 16:38:07 2001
+++ ./access.h	Sat Feb  8 23:05:27 2003
@@ -14,7 +14,7 @@
 #include "cache.h"
 #include "vmstime.h"
 
-#ifdef BIG_ENDIAN
+#ifdef USE_BIG_ENDIAN
 #define VMSLONG(l) ((l & 0xff) << 24 | (l & 0xff00) << 8 | (l & 0xff0000) >> 8 | l >> 24)
 #define VMSWORD(w) ((w & 0xff) << 8 | w >> 8)
 #define VMSSWAP(l) ((l & 0xff0000) << 8 | (l & 0xff000000) >> 8 |(l & 0xff) << 8 | (l & 0xff00) >> 8)

diff -u o/makefile.unix ./makefile.unix
--- o/makefile.unix	Thu Sep 27 02:40:14 2001
+++ ./makefile.unix	Sat Feb  8 23:06:24 2003
@@ -1,12 +1,12 @@
 
-CCFLAGS = "-g"
+CCFLAGS = -O4
 
 DEFS = "-DVERSION=\"v1.3\""
 
 all : ods2
 
 ods2 : ods2.o rms.o direct.o update.o access.o device.o phyunix.o cache.o vmstime.o
-	cc $(CCFLAGS) -oods2 ods2.o rms.o direct.o update.o access.o device.o phyunix.o cache.o vmstime.o
+	cc $(CCFLAGS) -o ods2 ods2.o rms.o direct.o update.o access.o device.o phyunix.o cache.o vmstime.o
 
 vmstime.o : vmstime.c vmstime.h
 	cc -c $(CCFLAGS) $(DEFS) vmstime.c

diff -u o/ods2.c ./ods2.c
--- o/ods2.c	Fri Sep  7 23:17:04 2001
+++ ./ods2.c	Tue Aug 10 22:43:23 2004
@@ -1,5 +1,7 @@
 #define MODULE_NAME	ODS2
-#define MODULE_IDENT	"V1.3"
+#define MODULE_IDENT	"V1.3hb"
+
+/*	Jul-2003, v1.3hb, some extensions by Hartmut Becker */
 
 /*     Ods2.c v1.3   Mainline ODS2 program   */
 
@@ -315,7 +317,7 @@
         nam.nam$b_rss = NAM$C_MAXRSS;
         fab.fab$l_fop = FAB$M_NAM;
         while ((sts = sys_search(&fab)) & 1) {
-            sts = sys_open(&fab);
+	    sts = sys_open(&fab);
             if ((sts & 1) == 0) {
                 printf("%%COPY-F-OPENFAIL, Open error: %d\n",sts);
                 perror("-COPY-F-ERR ");
@@ -1004,27 +1006,40 @@
 
 /* cmdsplit: break a command line into its components */
 
+/* 
+ * New feature for Unix: '//' stops qualifier parsing (similar to '--' 
+ * for some Unix tools). This enables us to copy to Unix directories.
+ * 	copy /bin // *.com /tmp/*
+ * is split into argv[0] -> "*.com" argv[1] -> "/tmp/*" qualv[0] -> "/bin"
+ */
+
 int cmdsplit(char *str)
 {
     int argc = 0,qualc = 0;
     char *argv[32],*qualv[32];
     char *sp = str;
     int i;
+    char q = '/';
     for (i = 0; i < 32; i++) argv[i] = qualv[i] = "";
     while (*sp != '\0') {
         while (*sp == ' ') sp++;
         if (*sp != '\0') {
-            if (*sp == '/') {
+            if (*sp == q) {
                 *sp++ = '\0';
+		if (*sp == q) {
+			sp++;
+			q = '\0';
+			continue;
+		}
                 qualv[qualc++] = sp;
             } else {
                 argv[argc++] = sp;
             }
-            while (*sp != ' ' && *sp != '/' && *sp != '\0') sp++;
+            while (*sp != ' ' && *sp != q && *sp != '\0') sp++;
             if (*sp == '\0') {
                 break;
             } else {
-                if (*sp != '/') *sp++ = '\0';
+                if (*sp != q) *sp++ = '\0';
             }
         }
     }
@@ -1073,13 +1088,65 @@
 
 /* main: the simple mainline of this puppy... */
 
+/*
+ * Parse the command line to read and execute commands:
+ * 	./ods2 mount scd1 $ set def [hartmut] $ copy *.com $ exit
+ * '$' is used as command delimiter because it is a familiar character
+ * in the VMS world. However, it should be used surounded by white spaces;
+ * otherwise, a token '$copy' is interpreted by the Unix shell as a shell
+ * variable. Quoting the whole command string might help:
+ * 	./ods2 'mount scd1 $set def [hartmut] $copy *.com $exit'
+ * If the ods2 reader should use any switches '-c' should be used for
+ * the command strings, then quoting will be required:
+ * 	./ods2 -c 'mount scd1 $ set def [hartmut] $ copy *.com $ exit'
+ *
+ * The same command concatenation can be implemented for the prompted input.
+ * As for indirect command files (@), it isn't checked if one of the chained
+ * commands fails. The user has to be careful, all the subsequent commands
+ * are executed!
+ */
+
 int main(int argc,char *argv[])
 {
     char str[2048];
+    char *command_line = NULL;
     FILE *atfile = NULL;
+    char *p;
     printf(" ODS2 %s\n", MODULE_IDENT);
+    if (argc>1) {
+	    int i, l= 0;
+	    for (i=1; i<argc; i++) {
+		    if (command_line==NULL) {
+			    command_line=malloc(1);
+			    *command_line = '\0';
+			    l= 1;
+		    }
+		    l+= strlen(argv[i]);
+		    command_line= realloc(command_line,l+1);
+		    strcat (command_line, argv[i]);
+		    command_line[l-1]= ' ';
+		    command_line[l++]= '\0';
+	    }
+	    if (l>1)
+		    command_line[l-2]= '\0';
+    } else command_line==NULL;
     while (1) {
         char *ptr;
+	if (command_line) {
+		static int i=0;
+		int j=0;
+		for (; j<sizeof str && command_line[i]; i++)
+			if (command_line[i]=='$') {
+				++i;
+				break;
+			} else str[j++]= command_line[i];
+		if (j<sizeof str)
+			str[j]= '\0';
+		else str[j-1]= '\0';
+		if (command_line[i]=='\0')
+			command_line= realloc(command_line,0);
+	} else
+{
         if (atfile != NULL) {
             if (fgets(str,sizeof(str),atfile) == NULL) {
                 fclose(atfile);
@@ -1098,7 +1165,8 @@
             if (gets(str) == NULL) break;
 #endif
         }
-        ptr = str;
+}
+	ptr = str;
         while (*ptr == ' ' || *ptr == '\t') ptr++;
         if (strlen(ptr) && *ptr != '!') {
             if (*ptr == '@') {

diff -u o/phyunix.c ./phyunix.c
--- o/phyunix.c	Thu Sep 27 11:00:42 2001
+++ ./phyunix.c	Sat Feb  8 23:06:54 2003
@@ -15,7 +15,7 @@
 */
 
 #include <stdio.h>
-#include <strings.h>
+#include <string.h>
 #include <unistd.h>
 #include <fcntl.h>
 
diff -u o/update.c ./update.c
--- o/update.c	Thu Sep 27 08:52:06 2001
+++ ./update.c	Sat Feb  8 23:05:50 2003
@@ -19,7 +19,7 @@
 /* Bitmaps get accesses in 'WORK_UNITs' which can be an integer
    on a little endian machine but must be a byte on a big endian system */
 
-#ifdef BIG_ENDIAN
+#ifdef USE_BIG_ENDIAN
 #define WORK_UNIT unsigned char
 #define WORK_MASK 0xff
 #else

